package guru.kake.ronove

import guru.kake.xmlp.XMLParser
import java.io.FileInputStream
import javax.xml.namespace.QName
import javax.xml.stream.XMLEventReader
import javax.xml.stream.XMLInputFactory
import javax.xml.stream.events.XMLEvent



class Character(var literal: String = "") : Comparable<Character> {
    var rmgroups: MutableList<RMGroup> = mutableListOf()

    fun print() {
        println("=== Character: $literal ===")
        for (n in rmgroups) {
            n.print()
        }
    }

    override fun equals(other: Any?): Boolean {
        if (other is Character) {
            return this.literal == other.literal
        }
        return super.equals(other)
    }

    override fun compareTo(other: Character) = this.literal.compareTo(other.literal)
    override fun hashCode(): Int = literal.hashCode()

}

class RMGroup {
    var reading: MutableList<Pair<String?, String>> = mutableListOf()
    var meaning: MutableList<Pair<String?, String>> = mutableListOf()

    fun print() {
        println("-- RMGroup --")
        for  (r in reading) {
            println("Reading: ${r.second} (${r.first})")
        }
        for  (m in meaning) {
            println("Meaning: ${m.second} (${m.first})")
        }
    }
}

class Phrase(var japanese: String) : Comparable<Phrase> {
    var senses: MutableList<Sense> = mutableListOf()

    override fun equals(other: Any?): Boolean {
        if (other is Phrase) {
            return this.japanese == other.japanese
        }
        return super.equals(other)
    }
    override fun compareTo(other: Phrase) = this.japanese.compareTo(other.japanese)
    override fun hashCode(): Int = japanese.hashCode()
}
class Sense {
    var glosses: MutableList<String> = mutableListOf()
}

class DictionaryParser {
    companion object {
        private val xmlInputFactory: XMLInputFactory = XMLInputFactory.newInstance()

        fun parseKanjidic2(path: String): List<Character> {
            val parsed: MutableList<Character> = mutableListOf()
            var character: Character? = null
            var rmgroup: RMGroup? = null

            XMLParser.parse(path) {
                element("character") {
                    start { character = Character() }
                    end {
                        parsed.add(character!!)
                        character = null
                    }
                }
                element("literal") {
                    start { character!!.literal = content()!! }
                }
                element("reading_meaning") {
                    start { rmgroup = RMGroup() }
                    end {
                        character!!.rmgroups.add(rmgroup!!)
                        rmgroup = null
                    }
                }
                element("meaning") {
                    start {
                        rmgroup!!.meaning.add(Pair(
                            attribute("m_lang"),
                            content()!!
                        ))
                    }
                }
                element("reading") {
                    start {
                        rmgroup!!.reading.add(Pair(
                            attribute("r_type"),
                            content()!!
                        ))
                    }
                }
            }
            return parsed.toList()
        }

        fun parseJMdict(path: String): List<Phrase> {
            //TODO: DSL
            val parsed: MutableList<Phrase> = mutableListOf()
            val phrases: MutableList<Phrase> = mutableListOf()
            var sense: Sense? = null

            val reader: XMLEventReader = xmlInputFactory.createXMLEventReader(FileInputStream(path))

            while (reader.hasNext()) {
                val nextEvent: XMLEvent = reader.nextEvent()
                when {
                    nextEvent.isStartElement -> {
                        val startElement = nextEvent.asStartElement()
                        when (startElement.name.localPart.toString()) {
                            //"entry" -> phrase = Character()
                            "keb" -> {
                                phrases.add(Phrase(reader.nextEvent().asCharacters().data))
                            }

                            "reb" -> {

                            }

                            "sense" -> {
                                sense = Sense()
                            }

                            "gloss" -> {
                                sense!!.glosses.add(reader.nextEvent().asCharacters().data)
                            }
                        }
                    }

                    nextEvent.isEndElement -> {
                        when (nextEvent.asEndElement().name.localPart.toString()) {
                            "entry" -> {
                                parsed.addAll(phrases)
                                phrases.clear()
                            }

                            "sense" -> {
                                phrases.forEach { it.senses.add(sense!!) }
                            }
                        }
                    }
                }
            }
            return parsed.toList()
        }
    }
}